/* dspy.h
 *
 * Copyright 2019 Christian Hergert <chergert@redhat.com>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#pragma once

#define DSPY_INSIDE

# include "dspy-connection.h"
# include "dspy-connection-button.h"
# include "dspy-introspection-model.h"
# include "dspy-method-invocation.h"
# include "dspy-method-view.h"
# include "dspy-name.h"
# include "dspy-name-marquee.h"
# include "dspy-name-row.h"
# include "dspy-names-model.h"
# include "dspy-tree-view.h"
# include "dspy-version.h"
# include "dspy-version-macros.h"
# include "dspy-view.h"

#undef DSPY_INSIDE
